#!/usr/bin/env node

const c = {
	BLACK: '\x1b[0m',
	GREEN: '\x1b[32m',
	RED: '\x1b[31m',
	YELLOW: '\x1b[33m',
	CYAN: '\x1b[36m',
}

let shuttlePkgJsonDir;
let shallRunInstall = true;

function _copyHandler(src, dest) {
	const fs = require("fs");
	const path = require("path");

	/* 
		TODO :: Prompt user => Directory not empty. Continue?
	*/
	if (fs.existsSync(dest) === false && src.indexOf("node_modules") === -1) 
		fs.mkdirSync(dest);

	let walked = fs.readdirSync(src);
	// Just in case, for some reason, we did have "node_modules", don't include it.
	walked = walked.filter(ele => ele.indexOf("node_modules") === -1)
	

	for(let i = 0; i < walked.length; i++) {
		let current = fs.lstatSync(path.join(src, walked[i]));

		if(current.isDirectory()) 
			_copyHandler(path.join(src, walked[i]), path.join(dest, walked[i]));

		else if(current.isSymbolicLink()) {
			let symlink = fs.readlinkSync(path.join(src, walked[i]));
			fs.symlinkSync(symlink, path.join(dest, walked[i]));
		} 
		else
			fs.copyFileSync(path.join(src, walked[i]), path.join(dest, walked[i]));
	}
}



function _copyShuttle(projectName) {
	const path = require("path");
	/* TODO :: show inspection :: 
		in case of permision denial <process.cwd()> can cause 
		<Error: ENOENT: no such file or directory, uv_cwd>  
	*/
	
	const projectDir = path.join(process.cwd(), projectName);
	const libDir = __dirname.replace('/bin', '/lib');
	_copyHandler(libDir, projectDir);

	shuttlePkgJsonDir = path.join(projectDir, 'package.json');
}

function _runNpmInstall() {
	console.log(`> ${c.YELLOW}running npm install...${c.BLACK}`);

	let { spawnSync } = require("child_process");
	let statusCode = spawnSync("npm", ["install", "--loglevel=error"], {cwd: shuttlePkgJsonDir.replace("package.json", "")});

	if (statusCode === 0) 
		return True
}


function successCheck(projectName) {
	let updatedPkgJson = require(shuttlePkgJsonDir);

	if(updatedPkgJson.name === projectName) {
		shallRunInstall ? _runNpmInstall() : null;
		console.log(`> ${c.GREEN}Success!${c.BLACK}`);
	}
}

function initializeProject(projectName) {
	if(projectName === undefined)
		console.log(`> ${c.RED}missing${c.BLACK} <project name> after 'initialize'\n`)
	else {
		// TODO :: show inspection (like possible reasons in case of crash)
		process.stdout.write(`> initializing project '${projectName}'... `);
		_copyShuttle(projectName);
		
		let shuttleProject = require(shuttlePkgJsonDir);
		let fs = require("fs");
		
		shuttleProject.name = projectName;
		fs.writeFileSync(shuttlePkgJsonDir, JSON.stringify(shuttleProject, null, 3));

		process.stdout.write(`${c.GREEN}done${c.BLACK}\n`);

		successCheck(projectName);
	}
}


let argv = require('yargs')
	.scriptName("react-shuttle")
	.parserConfiguration({ "boolean-negation": false, })
	.usage("Usage: $0 <Command> [Options?]")
	.command({
		command: "init <project-name>",
		aliases: ['initialize', 'i'],
		describe: "Create a new project at the current directory",
		nargs: 1,
		builder: (yargs) => yargs.positional("project-name", {
			demandOption: true,
			describe: "Name of the project ",
			type: "string",
		}),
	})
	.demandCommand(1, 'No commands specified')
	.options({
		'no-install': {
			alias: 'ni',
			describe: "DO NOT run 'npm install' after initialization",
			demandOption: false,
			default: false,
			type: 'boolean'
		}
	})
	.help('h').alias('help', 'h')
	.version()
	.argv;


shallRunInstall = (argv.noInstall ? false : true);
initializeProject(argv.projectName);