import { createStore, applyMiddleware, compose } from 'redux';
import thunkMiddleware from 'redux-thunk';
import { createLogger } from 'redux-logger';
import { routerMiddleware } from 'connected-react-router'
import { createBrowserHistory } from 'history'

import rootReducer from '../_redux/reducers';


const loggerMiddleware = createLogger({
  // Collapse the actions which don't have any errors.
  collapsed: (getState, action, logEntry) => !logEntry.error,
  duration: true,
  timestamp: true,
  diff: true,
});

export const history = createBrowserHistory()

/* 
  For universal "isomorphic" apps, prefix it with:
  >>> typeof window !== 'undefined' &&
  ...
*/
const composeEnhancers =
  typeof window === 'object' &&
  window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ ?   
    window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__({
      /* 
        Extension’s options. Visit the link below for the full API.
        https://github.com/zalmoxisus/redux-devtools-extension/blob/master/docs/API/Arguments.md
      */
      trace: true,
    }) : compose;

// Root reducer with router state.
const reducer = rootReducer(history);

const enhancers = composeEnhancers(
  applyMiddleware(
    thunkMiddleware,
    /* 
      For dispatching history actions.
      Can be easily used through @redux-thunk.
    */
    routerMiddleware(history),
    loggerMiddleware,
  ),
);


export default function configStore(preloadedState) {
  return createStore(reducer, preloadedState, enhancers);
}